// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "assert/assert.hpp"

namespace Dawn {
  typedef int32_t usagelockid_t;

  class UsageLock {
    protected:
      usagelockid_t internalPool = 0;
      usagelockid_t *pool = nullptr;
      std::vector<usagelockid_t> locks;

    public:
      std::function<void()> onEmpty;
      std::function<void()> onFirstLock;
      std::function<void(usagelockid_t)> onLockRemoved;

      /**
       * Construct a new usage lock object.
       */
      UsageLock();

      /**
       * Construct a new Usage Lock object
       * 
       * @param lockPool Pool that will be used to create locks.
       */
      UsageLock(usagelockid_t *lockPool);

      /**
       * Creates a new lock.
       * 
       * @return Lock created for this specific instance.
       */
      usagelockid_t createLock();

      /**
       * Removes a lock.
       * 
       * @param lock Lck to remove.
       */
      void removeLock(usagelockid_t lock);
  };
}