// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "asset/AssetLoader.hpp"
#include "util/mathutils.hpp"

using namespace Dawn;

AssetLoader::AssetLoader(std::string fileName) : IAssetLoader(fileName) {
  this->handle = nullptr;
}

void AssetLoader::open() {
  assertNull(this->handle, "AssetLoader::open: File is already open?");

  std::string pathFull = DAWN_ASSET_LOCATION + this->fileName;
  this->handle = fopen(pathFull.c_str(), "rb");
  fseek(this->handle, 0, SEEK_END);
  assertNotNull(this->handle, "AssetLoader::open: Failed to open file " + pathFull);

  this->size = ftell(this->handle);
  assertTrue(this->size > 0, "AssetLoader::open: File size is 0?");

  this->rewind();
}

int32_t AssetLoader::close() {
  assertNotNull(this->handle, "AssetLoader::close: File is not open");
  int32_t ret = fclose(this->handle);
  this->handle = nullptr;
  return ret;
}

size_t AssetLoader::read(uint8_t *buffer, size_t size) {
  assertNotNull(buffer, "AssetLoader::read: buffer must not be null");
  assertTrue(size > 0, "AssetLoader::read: size must be greater than 0");
  assertNotNull(this->handle, "AssetLoader::read: File is not open");
  return fread(buffer, 1, size, this->handle);
}

size_t AssetLoader::skip(size_t n) {
  assertTrue(n > 0, "AssetLoader::skip: n must be greater than 0");
  assertNotNull(this->handle, "AssetLoader::skip: File is not open");
  return fseek(this->handle, n, SEEK_CUR);
}

void AssetLoader::rewind() {
  assertNotNull(this->handle, "AssetLoader::rewind: File is not open");
  fseek(this->handle, 0, SEEK_SET);
}

size_t AssetLoader::getPosition() {
  assertNotNull(this->handle, "AssetLoader::getPosition: File is not open");
  return ftell(this->handle);
}

size_t AssetLoader::getSize() {
  return this->size;
}

AssetLoader::~AssetLoader() {
  if(this->handle != nullptr) {
    this->close();
    this->handle = nullptr;
  }
}